#!/usr/bin/python

# ZPhotoSlides, Zope product
# Copyright (C) 2002-2003  Jean-Francois Roche <jfroche@jfroche.be>
#                          Florent Carlier <florent.carlier@insa-rennes.fr>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# $Id: FS_files.py,v 1.9 2004/03/12 20:41:22 jfroche Exp $'

import os, sys, time
from Persistence import Persistent
import Acquisition
import AccessControl.Role
import OFS.SimpleItem
import AccessControl
from OFS.Folder import Folder
import AccessControl.Owned
from types import StringType
from OFS.ObjectManager import ObjectManager
class ZFile(Folder,OFS.SimpleItem.Item,Persistent,Acquisition.Implicit,AccessControl.Owned.Owned,AccessControl.Role.RoleManager,ObjectManager):
        """a new file/folder on fs"""
        access=AccessControl.getSecurityManager()
        security = AccessControl.ClassSecurityInfo()
                   
	def __init__(self,idname, filePath, uploaded,recursive):
		self.id = idname
		self.filePath = os.path.abspath(os.path.normpath(filePath))
		self.fullpath = filePath
		self.basepath = filePath
		self.pathExist = os.path.exists(self.filePath)
		self.recursive = recursive
                self.meta_type = 'ZFile'
                self.uploaded = uploaded
		if not self.pathExist:
#			 sys.exit("Path %s does not exist"%filePath)
                        raise IOError,"Path %s does not exist"%filePath

		self.fileName = os.path.basename(self.filePath)
                if(self.isFile()):
                    self.modif_date = self.getLastModif()
                    self.size = self.getSize()
                    self.file = 1
		if(self.isFile() and self.uploaded): # should we upload object in ZODB?
                    fd = open(self.fullpath,'rb')
                    self.data = fd.read()
                    fd.close()
		if not self.fileName:
			self.fileName = os.sep
                self.update_file_list()
                self._p_changed = 1
                        
        def update_file_list(self):
		if(self.isDir()):
			if(self.recursive):
				list = self.getRecursiveFiles()
			else:
				list = self.getFiles()
			for file in list:
				path1,path2 = os.path.split(file)
                                if(not hasattr(self,path2)) or ((hasattr(self,path2) and self.check_modified(path2))):
				    tmp = ZFile(file,file,self.uploaded,self.recursive)
				    setattr(self,path2,tmp)
                                    self._p_changed = 1

        def check_modified(self,filename2):
            file = getattr(self,filename2)
            date = file.getLastModif()
            if(not hasattr(self,'modif_date')):
                file.modif_date = file.getLastModif()
                return 0
            if(not hasattr(self,'size')):
                file.size = file.getSize()
                return 0
            if(date != file.modif_date or file.size != file.getSize()): # file not same date or not same size then modified
                return 1
            else:
                return 0

        def fileIds(self):
            return self.getFile()
	def getFile(self):
                "file IDS"
		if self.isDir():
			fileList = []
			rawFileList = []
			rawFileList = os.listdir(self.filePath)
			for file in rawFileList:
				if os.path.isfile(os.path.join(self.filePath, file)):
					fileList.append(file)
			return fileList
		else:
			return None
		
	def getParent(self):
		if os.path.dirname(self.filePath) == self.filePath:
			self.parentDir = None
		else:
			self.parentDir = os.path.dirname(self.filePath)
		return self.parentDir
		
	def isDir(self):# est-ce un repertoire ?
		if os.path.isdir(self.filePath):
			return 1
		else:
			return 0
	
	def isFile(self):# est-ce un fichier ?
		if os.path.isfile(self.filePath):
			return 1
		else:
			return 0
	
	def getDir(self):
		if self.isDir():
			fileList = []
			dirList = []
			fileList = os.listdir(self.filePath)
			for file in fileList:
				if os.path.isdir(os.path.join(self.filePath, file)):
					dirList.append(os.path.join(self.filePath, file))
			return dirList
		else:
			return None

	def getFiles(self):
		if self.isDir():
			fileList = []
			rawFileList = []
			rawFileList = os.listdir(self.filePath)
			for file in rawFileList:
				if os.path.isfile(os.path.join(self.filePath, file)) or file == 'new':
				        fileList.append(os.path.join(self.filePath, file))
			return fileList
		else:
			return None

	def visit(self, arg, dirName, files):
		self.recursiveDirList.append(dirName)
	
	def getRecursiveDirs(self):
		if self.isDir():
			self.recursiveDirList = []
			os.path.walk(self.filePath, self.visit, None)
			return self.recursiveDirList
		return None

	def getRecursiveFiles(self):
		if self.isDir():
			self.recursiveFileList = []
			dirList = self.getRecursiveDirs()
			for dir in dirList:
				files = os.listdir(dir)
				for file in files:
					#if os.path.isfile(os.path.join(dir, file)):
					self.recursiveFileList.append(os.path.join(dir, file))
			return self.recursiveFileList
		return None

	def getStat(self):
		stat_list = []
		stat_list = os.stat(self.filePath)
		return stat_list
	
	def getLastAccess(self):
		a = self.getStat()
		return time.ctime(a[7])

	def _getOb(self,file):
		if(self.isDir()):
			obj = getattr(self,file)
			return obj

        def index_html(self):
                "default view"
                if(hasattr(self,'file')):
                    return self.GET()
                elif(self.isFile()):
                    return self.GET()
                else:
                    raise IOError,"no access to this folder!"

	def GET(self):
		if(hasattr(self,'file') and not self.uploaded):
			fd = open(self.fullpath,'rb')
			data = fd.read()
                        fd.close()
                        return data
                elif(hasattr(self,'file') and self.uploaded):
                        return self.data
                else:
                        return None

	
	def getLastModif(self):
		a = self.getStat()
		return time.ctime(a[8])
	
	def getLastStatus(self):
		a = self.getStat()
		return time.ctime(a[9])
	
	def get_size(self):
		return self.getSize()

	def getSize(self):
		if self.isFile():
			a = self.getStat()
			return a[6]
#		else:
#			dirSize = 0
#			for i in self.getBiggerFileList():
#				dirSize = dirSize + i[1]
#			return dirSize

	def cmpSize(self, first, second):
		if int(first[1]) < int(second[1]):
			return 1
		elif int(first[1]) == int(second[1]):
			return 0
		else:
			return -1

	def __str__(self):
		filepath = self.filePath
		parentdir = self.getParent()
		filesize = self.getSize()
		isfile = self.isFile()
		isdir = self.isDir()
		access_modif_status = (self.getLastAccess(), self.getLastModif(), self.getLastStatus())
		currentdirlist = self.getDir()
		currentfiles = self.getFiles()
		alldirs = self.getRecursiveDirs()
		allfiles = self.getRecursiveFiles()
		fileIdentity = "filepath : %s\nparent dir: %s\nfile size : %s bytes\nfile? %s\ndir ? %s\n"%(filepath, parentdir, filesize, isfile, isdir)
		curdirliststring, curfileliststring, alldirstring, allfilestring = "", "", "", ""
		if self.isDir():
			for dir in currentdirlist:
				curdirliststring = curdirliststring + dir + "\n"
			for file in currentdirlist:
				curfileliststring = curfileliststring + file + "\n"
			for dir in alldirs:
				alldirstring = alldirstring + dir + "\n"
			for file in allfiles:
				allfilestring = allfilestring  + file + "\n"
		else:
			curdirliststring, curfileliststring, alldirstring, allfilestring = None, None, None, None
		mode = "last access : %s\nlast modification : %s\nlast status : %s\n"%(access_modif_status[0], access_modif_status[1], access_modif_status[2])
		listings = "dirs : %s\nfiles : %s\nall dirs : %s\nall files : %s\n"%(curdirliststring, curfileliststring, alldirstring, allfilestring)
		objectString = "%s%s%s"%(fileIdentity, mode, listings)
		return objectString
		
def main():
	try:
		path = sys.argv[1]
	except IndexError:
		path = os.curdir
	
	try:
		col = int(sys.argv[2])
	except IndexError:
		col = 50
	
	a = FileObj('img',path, 0,col)
	#print a.__dict__
	#a.printBiggerFileList()
	#print a._getOb('CHANGES')
	
if __name__ == "__main__":
	main()
