# ZPhotoSlides, Zope product
# Copyright (C) 2002-2003  Jean-Francois Roche <jfroche@jfroche.be>
#                          Florent Carlier <florent.carlier@insa-rennes.fr>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# $Id: ZPhotoSlidesFolder.py,v 1.46 2004/03/12 20:41:22 jfroche Exp $' 

"""ZPhotoSlides Folder
"""

VERSION = '0.5'
from Globals import Persistent
from Globals import package_home
from OFS.ObjectManager import ObjectManager
from OFS.SimpleItem import Item
from OFS.PropertyManager import PropertyManager
from OFS.FindSupport import FindSupport
from AccessControl.Role import RoleManager
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base
from Globals import MessageDialog
from Globals import InitializeClass, HTMLFile, HTML
from ZPhotoSlides import ZPhotoSlides
from Globals import DTMLFile
from Products.ZCatalog.ZCatalog import ZCatalog
import os
import string
from DateTime import DateTime
import ZPhotoSlidesUtils
from ZPSPloneBatch import Batch
import whrandom
import comment
import layout
import time
from stat import stat


# Protection Text : Strip-o-Gram HTML Conversion Library
try:
        from Products.stripogram import html2safehtml
except:
        have_stripogram = 0
        from Products.PythonScripts.standard import html_quote
        def html2safehtml(s, valid_tags=('b', 'a', 'i', 'br', 'p')):
	    return html_quote(s)
else:
        have_stripogram = 1

try:
	from Products.Localizer import LocalDTMLFile
	from Products.Localizer import LocalPageTemplateFile
	from Products.Localizer import Gettext
	from Products.Localizer import Accept
	from Products.Localizer import LanguageManager
	zpsgettext = Gettext.translation(globals())
except:
	have_localizer = 0
	from Globals import DTMLFile
	from Products.PageTemplates.PageTemplateFile import PageTemplateFile
	LocalPageTemplateFile = PageTemplateFile
	LocalDTMLFile = DTMLFile
	del PageTemplateFile
	del DTMLFile
else:
	have_localizer = 1

if(not have_localizer):
	from ZPhotoSlidesUtils import zpsgettext
_ = zpsgettext

class ZPhotoSlidesFolder(
                         ObjectManager,
                         PropertyManager,
                         RoleManager,
                         Item,
                         FindSupport):
    """ZPhotoSlides Folder object implementation."""
    if(have_localizer):
    	locale = package_home(globals()) + '/locale/'
	_zpsgettext = zpsgettext

    manage_options = (
                       {'label': "Manage ZPhotoSlides",    'action':'manage_main'},
		       {'label': "View",	'action': 'index_html'},
	               {'label': "Properties",       'action': 'manage_props'},
                       {'label': "Edit layout", 'action': 'user_layout_folder'},
		       {'label': "Manage positions",'action': 'manage_position'},
		       {'label': "Manage comments",'action': 'manage_comment'},
		       {'label': "Search for photos",'action': 'search_photos'},
		       {'label': "Statistics",'action': 'stat_folder'},
                       {'label': "Security",   'action': 'manage_access'},
                       {'label': "Undo", 'action': 'manage_UndoForm'},
                       {'label': "Ownership", 'action': 'manage_owner'},
		       {'label': "About this product", 'action':'about'}
		     )
    security=ClassSecurityInfo()
    meta_type = "ZPhotoSlides Folder"
    # ZPT
    index_html_zpt = LocalPageTemplateFile('zpt/index_folder_html',globals(),gettext=zpsgettext)
    batch_macros = LocalPageTemplateFile('zpt/batch_macros',globals(),gettext=zpsgettext)
    footer = LocalPageTemplateFile('zpt/footer',globals(),gettext=zpsgettext)
    header = LocalPageTemplateFile('zpt/header',globals(),gettext=zpsgettext)
    breadcrumb = ZPhotoSlidesUtils.breadcrumb

    index_html = LocalDTMLFile('dtml/index_folder_html',globals(),zpsgettext=zpsgettext)
    css_info = HTMLFile('dtml/stylesheet_demo.css',globals(),zpsgettext=zpsgettext)
    manage_position = LocalDTMLFile('dtml/manage_ZPS_position',globals(),zpsgettext=zpsgettext)
    define_css = LocalDTMLFile('dtml/define_css',globals(),zpsgettext=zpsgettext)
    manage_comment = LocalDTMLFile('dtml/manage_folder_comment',globals(),zpsgettext=zpsgettext)
    edit_comment = LocalDTMLFile('dtml/edit_folder_comment',globals(),zpsgettext=zpsgettext)
    view_folder_comment = LocalDTMLFile('dtml/view_folder_comment',globals(),zpsgettext=zpsgettext)
    add_folder_comment = LocalDTMLFile('dtml/add_folder_comment',globals(),zpsgettext=zpsgettext)
    manage_tabs = LocalDTMLFile('dtml/manage_tabs',globals(),zpsgettext=zpsgettext)
    manage_props = LocalDTMLFile('dtml/ZPhotoSlidesFolderEdit',globals(),zpsgettext=zpsgettext) # edit main properties
    user_layout_folder = LocalDTMLFile('dtml/user_layout_folder',globals(),zpsgettext=zpsgettext)
    plone_css = HTMLFile('dtml/plone_base_css',globals(),zpsgettext=zpsgettext)
    about = LocalDTMLFile('dtml/about',globals(),zpsgettext=zpsgettext)                         # about us
    tree = LocalDTMLFile('dtml/zphoto_tree',globals(),zpsgettext=zpsgettext)
    css_file = HTMLFile('dtml/stylesheet.css',globals(),zpsgettext=zpsgettext) 
    search_photos = LocalDTMLFile('dtml/search_photo',globals(),zpsgettext=zpsgettext)
    search_result = LocalDTMLFile('dtml/search_result',globals(),zpsgettext=zpsgettext)
    manage_main = LocalDTMLFile('dtml/list_zphotoslides',globals(),zpsgettext=zpsgettext)
    stat_folder = LocalDTMLFile('dtml/stat_folder',globals(),zpsgettext=zpsgettext)
    security.declarePublic('index_html_zpt','tree','index_html','footer','header','breacrumb','batch_macros','css_file','search_photos','search_result')
    security.declareProtected('manage_position','manage_tabs','manage_props','user_layout_folder','about','manage_main','stat_folder')
    _properties = (
        {'id':'title', 'type': 'string', 'mode': 'w'},
        )


    def __init__(self, id, title, description,position):

        self.version = VERSION
        self.id = id
        self.title = title
	self.position = position
	self.show_breadcrumbs = 1
        self.actual_css = 'stylesheet_default_css'
        self.layout_list = layout.give_list()
        self.update_date = '12/03/2004'
        self.zphoto_header_folder= ''                  # get content of standard header Folder
        self.zphoto_footer_folder= ''                  # get content of standard header Folder
        self.keywords = ''                      # keywords describing the object
        self.description = HTML(description)                   # description of the object
        self.manage_addFolder('css', 'Thumbnails', createPublic=0, createUserF=0, REQUEST=None)
	catalog = ZCatalog('catalog',container=self)
	self._setObject('catalog', catalog)
	self.manage_catalog()
	self.css.manage_addDTMLMethod('stylesheet_base_css', 'Main ZPhotoSlidesFolder Stylesheet', self.css_file, REQUEST=None)
        self.css.manage_addDTMLMethod('plone_css', 'Plone ZPhotoSlides Stylesheet', self.plone_css, REQUEST=None)
        self.css.manage_addDTMLMethod('stylesheet_default_css', 'Default ZPhotoSlides Stylesheet', self.css_info, REQUEST=None)
	self.last_modif = DateTime()
	self.show_thecomment = 1
	self.add_thecomment = 1
	self.list_comment  = comment.comments()

    security.declareProtected('View', 'getBatch')
    def getBatch(self, files, b_size, b_start, orphan):
      """ Patch ZPT """
      return Batch(files, b_size, int(b_start), orphan=0)

    security.declareProtected('View management screens','manage_catalog')
    def manage_catalog(self):
	try:    #for 2.5 ZCatalog which add automaticaly id and title and bobobase_modification_time
		self.catalog.manage_delIndex(ids=['id','title','bobobase_modification_time'])
	except:
		pass
	self.catalog.addIndex('id','FieldIndex')
	self.catalog.addIndex('title','TextIndex')
	self.catalog.addIndex('keywords','TextIndex')
	try:
		self.catalog.addIndex('bobobase_modification_time','DateIndex')
	except:	#zope 2.5 has no DateIndex =(
		self.catalog.addIndex('bobobase_modification_time','FieldIndex')
 
    security.declareProtected('View management screens','change_order_by_date')
    def reCatalog(self, REQUEST=None):
	"""clear the catalog, and put all images into it."""
        self.catalog._catalog.clear()
        path='/'+string.split(REQUEST.URL1, REQUEST.SERVER_URL)[1][1:]
	self.catalog.ZopeFindAndApply(self, obj_metatypes=['Image'], search_sub=1,REQUEST=REQUEST, apply_func=self.catalog.catalog_object, apply_path=path)
	return self.search_photos(manage_tabs_message="Objects recataloged!")

    security.declareProtected('View management screens','upgrade_zphotoslides')
    def upgrade_zphotoslides(self):
	"upgrade all ZPhotoSlides inside"
	for zphoto in self.objectValues('ZPhotoSlides'):
		zphoto.upgrade()
	for album in self.objectValues('ZPhotoSlides Folder'):
		album.upgrade_zphotoslides()
	return self.manage_props(manage_tabs_message=_('ZPhotoSlides Updated'))

    security.declareProtected('View management screens','change_order_by_date')
    def change_order_by_date(self,REQUEST=None):
	"update position of the different zphotoslides and zps fallowing the date of modification"
	list_zp = self.objectValues(['ZPhotoSlides','ZPhotoSlides Folder'])
	list_zp.sort(lambda x, y: cmp(x.bobobase_modification_time,y.bobobase_modification_time))
	cpt = 1
	for item in list_zp:
		item.position = cpt
		cpt = cpt + 1
        if(REQUEST!=None):
	    return self.manage_position(manage_tabs_message=_("Positions updated!"))

    security.declareProtected('View management screens', 'change_order')
    def change_order(self,REQUEST):
	"change the order of the zphotoslides and zps"
 	for zphoto_item in self.objectValues(['ZPhotoSlides','ZPhotoSlides Folder']):
		item = zphoto_item.getId() + '_position'
		for key in REQUEST.form.keys():
			if(item == key):
				zphoto_item.position = int(REQUEST.form[key])
	return self.manage_position(manage_tabs_message="Positions updated!",zpsgettext=zpsgettext)

    security.declareProtected('View management screens','needupgrade')
    def needupgrade(self):
	if(self.version in ['0.2','0.3','0.4']):
		return 1
	else:
		return 0

    security.declareProtected('View management screens', 'upgrade')
    def upgrade(self,REQUEST):
	"Allow the user to Upgrade to the new ZPhotoSlides Folder"
        attr_list = [ ("zphoto_header_folder", ''),
	              ("zphoto_footer_folder", ''),
		      ("keywords", ''),
		      ("position", len(self.aq_parent.objectValues(['ZPhotoSlides','ZPhotoSlides Folder']))),
		      ("show_breadcrumbs", 1),
		      ("last_modif", DateTime()),
		      ("show_thecomment", 1),
		      ("add_thecomment", 1),
		      ("list_comment", comment.comments()),
                      ("layout_list",layout.give_list()),
	              ("visits_log", stat())
		      ]

        for attr,default_val in attr_list:
	        if not self.__dict__.has_key(attr):
		       setattr(self,attr,default_val)
		       
        if(self.version == '0.2'):
		try:
			del self.index_html
		except:
			pass
		self.version = '0.3'
	
	if(self.version == '0.3'):
		#add catalog
		try:
			catalog = ZCatalog('catalog',container=self)
			self._setObject('catalog',catalog)
			self.manage_catalog()
			self.reCatalog(REQUEST)
		except:
			pass
		self.version = '0.4'
        # ZPhotoSlidesFolder upgraded!
	self.version = VERSION
        return self.manage_props(REQUEST,manage_tabs_message='ZPhotoSlides Folder Upgraded!')

    security.declareProtected('Access contents information', 'zphotoslidesIds')
    def manage_edit(self, title,description,show_breadcrumbs,add_thecomment,show_thecomment,REQUEST):
        """Edit the main properties..."""
        self.title = title
	if(description!=''):
	      self.description = HTML(description)
	else:
	      self.description = ''
        print "description"
        print self.description
	self.show_breadcrumbs = show_breadcrumbs
        self.show_thecomment = int(show_thecomment)
	self.add_thecomment = int(add_thecomment)
	self.last_modif = DateTime()
        if REQUEST is not None:
            return MessageDialog(
                title = _("Edited"),
                message = _("Properties for") + " %s " % self.id + _("changed"),
                action = './manage_props',
                )

    security.declarePublic('use_comment')
    def use_comment(self,id_folder,name,commt,place,email,REQUEST):
	"add a new comment for id_folder"
	times = time.strftime('%H:%M:%S %d-%m-%Y', time.localtime()) # when the comment is add
	clean_commt = html2safehtml(commt, valid_tags=('b','a','i','br','p'))
        iplog = self.visits_log.ip_log(REQUEST)
        self.list_comment.add_new(id_folder,clean_commt,name,place,times,iplog,email)
        if REQUEST is not None:
            return MessageDialog(
                title = _("Comment Added"),
                message = _("Comment in ") + " %s " % id_folder + _("added"),
                action = './index_html',
                )

    security.declareProtected('View management screens','use_uploaded_css')
    def use_uploaded_css(self,REQUEST):
	"Use a css which is in folder css"
	choosed = REQUEST.uploaded_css
	self.actual_css = choosed
	if(REQUEST!=None):
                return self.user_layout_folder(manage_tabs_message='The Css file you choose is now used to define the layout !')

    security.declareProtected('View management screens','add_new_css')
    def add_new_css(self,REQUEST):
	"Add a new CSS file in css folder"
	title=REQUEST.title
	file = REQUEST.file
	id = 'css' + str((len(self.css.objectValues())))
	self.css.manage_addDTMLMethod(id, title, file, REQUEST=None)
	self.actual_css = id
	if(REQUEST!=None):
		return self.user_layout_folder(manage_tabs_message='Your CSS file has been added !')

    security.declareProtected('View management screens','change_main_css')
    def change_main_css(self,REQUEST):
	"Change the main css tag for the Main ZPhotoSlides Stylesheet"
	dict = REQUEST.form
	for elem in dict.keys():
	   dic1,dic2 = string.split(elem,'__')
	   self.layout_list[dic1][dic2] = dict[elem]
	self.actual_css = 'stylesheet_base_css'
	if(REQUEST!=None):
		return self.user_layout_folder(manage_tabs_message='Modifications saved for Main ZPhotoSlides Stylesheet!')
		
    security.declareProtected('View management screens','modify_commentaire')    
    def modify_commentaire(self,photo_id,old_date,date,name,email,place,commt,REQUEST):
	"Manager modify a comment"
	for comment in self.list_comment:
		if(comment.id==photo_id and comment.date==old_date):
			comment.date = date
			comment.user = name
			comment.email = email
			comment.place = place
			comment.commt = commt
			break
		
	if REQUEST is not None:
		return self.manage_comment(manage_tabs_message=_('Comment Modified for ')+ str(photo_id))
		
    security.declareProtected('View management screens','delete_commentaire')
    def delete_commentaire(self,photo_id,comment_date,REQUEST):
	"Delete a comment from a photo fallowing the comment date"
	for comment in self.list_comment:
		if(comment.id==photo_id and comment.date==comment_date):
			self.list_comment.remove(comment)
			break
	if REQUEST is not None:
		return self.manage_comment(manage_tabs_message=_('Comment deleted for')+ str(photo_id))
		
    def number_of_commentaire(self,id_photo):
	"Returns the number of comments"
	count = 0
	for elem in self.list_comment:
		if(elem.id == id_photo):
			count = count + 1
	return count

    security.declarePublic('get_commentaire')    
    def get_commentaire(self,id_photo):
	"Print the comments for a photo"
        liste_dict = []                              # new list
        for elem in self.list_comment:               # for all comments
            if(elem.id==id_photo):                   #   if comment for the photo
                    dict= {}                         #       we add the comment to the list
                    dict['comment'] =elem.commt
                    dict['user'] = elem.user
                    dict['place'] = elem.place
                    dict['date'] = elem.date
                    dict['ip'] = elem.ip
                    dict['email'] = elem.email
                    liste_dict.append(dict)
        return liste_dict                               # we return the list

    security.declareProtected('Access contents information', 'zphotoslidesIds')
    def zphotoslidesIds(self):
        """Return list of ZPhotoSlides in this folder."""
        return self.objectIds(['ZPhotoSlides'])

    security.declareProtected('Access contents information', 'get_all_photos')
    def get_all_photos(self):
	list = []
	for zphoto in self.objectValues('ZPhotoSlides'):
		list = list + zphoto.thumbnails.objectValues('Image')
	for folder in self.objectValues('ZPhotoSlides Folder'):
		list = list + folder.get_all_photos()
	return list

    security.declareProtected('Access contents information', 'numZPhotoSlides')
    def numZPhotoSlides(self):
        """Return number of ZPhotoSlides in folder tree."""
        ZPhotoSlides = len(self.objectIds(['ZPhotoSlides']))
        for folder in self.objectValues(['ZPhotoSlides Folder']):
            ZPhotoSlides = ZPhotoSlides + folder.numZPhotoSlides()
        return ZPhotoSlides

    security.declareProtected('View','numZPhotoSlidesFolder')
    def numZPhotoSlidesFolder(self):
	nbr = len(self.objectIds(['ZPhotoSlides Folder']))
	for folder in self.objectValues(['ZPhotoSlides Folder']):
            nbr = nbr + folder.numZPhotoSlidesFolder()
	return nbr

    security.declareProtected('View','numZPhotoSlidesFolder')
    def get_thumb_random(self,thumb_album):
        photorandom = whrandom.choice(thumb_album.thumbnails.objectValues('Image'))
	return photorandom

    security.declareProtected('View','get_nbr_non_hidden_thumb')
    def get_nbr_non_hidden_thumb(self):
	"return the number of photos inside all the ZPhotoSlides of the folder"
	sum = 0
	for zphoto in self.objectValues('ZPhotoSlides'):
		sum = sum + len(zphoto.list_non_hidden_pic)
	for album in self.objectValues('ZPhotoSlides Folder'):
		sum = sum + album.get_nbr_non_hidden_thumb()
	return sum

    security.declareProtected('View','give_header_4_robots')
    def give_header_4_robots(self):
	"give the header for the robots"
	return """
<meta name="KEYWORDS" content="%s" />
<meta name="DESCRIPTION" content="%s" />
<meta name="resource-type" content="document" />
<meta name="ROBOTS" content="INDEX,FOLLOW" />
<meta http-equiv="expires" content="0" />
<meta name="author" content="ZPhotoSlides Folder" />
<meta name="copyright" content="ZPhotoSlides Folder %s" />
<meta name="revisit-after" content="1 days" />
<meta name="rating" content="General" />""" % (self.keywords,self.description,self.version)

    security.declareProtected('View management screens','new_header_footer_folder')
    def new_header_footer_folder(self,REQUEST):
	"Change the standard_html_header and footer Folder"
	self.zphoto_header_folder = REQUEST.header
	self.zphoto_footer_folder = REQUEST.footer
	if(REQUEST.header != '' or REQUEST.footer != ''):
                msg1 = _("""You use yet the standard html footer and header. To use the css files again
                make the standard html footer and header <u><i>EMPTY</i></u> in the layout formular!""")
                title_1 = "Using standard html header and footer"
        else:
		msg1 = _("You use yet the CSS file.")
		title_1 = _("Using CSS file")
	return self.REQUEST.RESPONSE.redirect('%s/user_layout_folder?message1=%s&message_title=%s' % (REQUEST.URL1, msg1, title_1))

    security.declarePublic('__len__')
    def __len__(self):
	return len(self.objectValues(['ZPhotoSlides','ZPhotoSlides Folder']))

    security.declarePublic('__getitem__')
    def __getitem__(self,i):
	list = self.objectValues(['ZPhotoSlides','ZPhotoSlides Folder'])
	list.sort(lambda x,y: cmp(x.position,y.position))
	return list[i]
    security.declarePublic('index')
    def index(self,item):
	list = self.objectValues(['ZPhotoSlides','ZPhotoSlides Folder'])
	list.sort(lambda x,y: cmp(x.position,y.position))
	return list.index(item)

manage_addZPhotoSlidesFolderForm = LocalDTMLFile('dtml/addZPhotoSlidesFolderForm', globals(),zpsgettext=zpsgettext)

def get_folder_list(folder): # get the list of folders
    pass
    

def manage_addZPhotoSlidesFolder(dispatcher, id, title,description,folder='', REQUEST=None):
    """Add ZPhotoSlides Folder object"""
    dest = dispatcher.Destination()
    position = len(dispatcher.objectValues(['ZPhotoSlides Folder','ZPhotoSlides'])) + 1
    dest._setObject(id, ZPhotoSlidesFolder(id, title,description,position))
    self = dest._getOb(id)
    if(folder!=''):
	if(not os.path.isdir(folder)):
		raise IOError,'Error: %s : folder unknown!!' % photo_folder
	else:
		pass	    

    if REQUEST is not None:
        try:    url=dispatcher.DestinationURL()
        except: url=REQUEST['URL1']
        REQUEST.RESPONSE.redirect('%s/%s/manage_props' % (url,id))
    return id

InitializeClass(ZPhotoSlidesFolder)
