##############################################################################
#
# Thanks to Fredrik Lundh for sharing the core IPTC reader. 
#
##############################################################################

from PIL import Image, TiffImagePlugin, JpegImagePlugin, IptcImagePlugin, TiffTags
import StringIO
import string 

            
li = [  ((2, 120), 'Caption'),
        ((2, 122), 'Caption_Writer'), 
        ((2, 105), 'Headline'), 
        ((2, 40), 'Special_Instructions'),
        ((2, 25), 'Keywords'),
        ((2, 15), 'Category'),
        ((2, 20), 'Supplemental_Categories'), 
        ((2, 10), 'Urgency'), 
        ((2, 80), 'Byline'), 
        ((2, 85), 'Byline_Title'),
        ((2, 110), 'Credit'),
        ((2, 115), 'Source'), 
        ((2, 5), 'Object_Name'),
        ((2, 55), 'Date_Created'),
        ((2, 90), 'City'), 
        ((2, 95), 'Province_State'),
        ((2, 101), 'Country_Name'),
        ((2, 103), 'Original_Transmission_Reference'),
        ((2, 116), 'Copyright_Notice')]

def _getfilteredIPTCfields(im):    
    """Returns the list of IPTC fields used by Photoshop, discarding other existing
    fields within the image object. """
    info = getiptcinfo(im)
    nli = []
    for k,v in li:
        try:
            rdic = str(info[k])
            del info[k]
        except:
            rdic = ''
        nli.append((v,rdic))    
    return nli

# read IPTC info fields from PhotoShop TIFF and JPEG files
#
# by Fredrik Lundh, November 2001

class FakeImage:
    pass

def getiptcinfo(im):
    """ extract IPTC data from a PhotoShop JPEG or TIFF file. """

    data = None

    if isinstance(im, JpegImagePlugin.JpegImageFile):
        # extract the IPTC/NAA resource
        try:
            app = im.app["APP13"]
            if app[:14] == "Photoshop 3.0\x00":
                app = app[14:]
                # parse the image resource block
                offset = 0
                while app[offset:offset+4] == "8BIM":
                    offset = offset + 4
                    # resource code
                    code = JpegImagePlugin.i16(app, offset)
                    offset = offset + 2
                    # resource name (usually empty)
                    name_len = ord(app[offset])
                    name = app[offset+1:offset+1+name_len]
                    offset = 1 + offset + name_len
                    if offset & 1:
                        offset = offset + 1
                    # resource data block
                    size = JpegImagePlugin.i32(app, offset)
                    offset = offset + 4
                    if code == 0x0404:
                        # 0x0404 contains IPTC/NAA data
                        data = app[offset:offset+size]
                        break
                    offset = offset + size
                    if offset & 1:
                        offset = offset + 1
        except (AttributeError, KeyError):
            pass

    elif isinstance(im, TiffImagePlugin.TiffImageFile):
        # get raw data from the IPTC/NAA tag (PhotoShop tags the data
        # as 4-byte integers, so we cannot use the get method...)
        try:
            type, data = im.tag.tagdata[TiffImagePlugin.IPTC_NAA_CHUNK]
        except (AttributeError, KeyError):
            pass

    if data is None:
        return None # no properties

    # create an IptcImagePlugin object without initializing it
    im = FakeImage()
    im.__class__ = IptcImagePlugin.IptcImageFile

    # parse the IPTC information chunk
    im.info = {}
    im.fp = StringIO.StringIO(data)

    try:
        im._open()
    except (IndexError, KeyError):
        pass # expected failure

    return im.info

if __name__ == "__main__":
	img = Image.open('/home/zphotoslides/zphotoslides_demo/enfant/DSC07525_iptc.JPG')
        b = getiptcinfo(img)
        print b
        a = _getfilteredIPTCfields(img)
        print a   
 
#                iptcinfo = _getfilteredIPTCfields(im)
#                for k, v in iptcinfo:
#                    if k == 'Date_Created':                       
#                        if len(v) == 8: fdate = v[:4]+'/'+v[4:6]+'/'+v[6:8] # back to Photoshop's style
#                        else: fdate = v                             
#                        setattr(self,'Date_Created', fdate)    # instead of the line below. Is it needed?               
#                    else:                        
#                        setattr(self,k,v)   # Puts the content of fields in the object's properties
